"use strict";

var game = game || {};

// loads sound and main
window.onload = function(){
    game.sound.init();
    game.main.sound = game.sound;
	game.main.init();
};

// Module that runs the game
game.main = {
    canvas: undefined,
    ctx: undefined,
    WIDTH: 0,
    HEIGHT: 0,

    backgroundMusic: "Dragon_and_Toast.mp3",
    fireballSound: "248116__robinhood76__05224-fireball-whoosh.mp3",
    castleAttackSound: "160045__jorickhoofd__metal-hit-with-metal-bar-resonance.mp3",
    eatingSound: "352207__inspectorj__crunching-wooden-fence-a.mp3",

    playerSprite: undefined,
    castleSprite: undefined,
    enemySprite: undefined,
    titleScreenImage: undefined,
    gameOverImage: undefined,

    grid: undefined,       // Find stuff with x*width + y
    gridWidth: 0,
    gridHeight: 0,
    spaceSize: 0,
    player: undefined,
    playerMovement: 0,
    playerSpriteY: 0,
    bulletList: undefined,
    fireball: undefined,
    fireSize: 0,
    GAME_STATE: Object.freeze({
        TITLE_SCREEN: 0,
        GAME: 1,
        GAME_OVER: 2
    }),
    gameState: undefined,
    paused: false,
    numPauseRect: 0,
    animationID: 0,
    button: undefined,
    castle: undefined,
    castleSize: 0,
    playerSize: 0,
    lastTime: 0,
    enemies: undefined,
    enemySize: 0,
    enemyCount: 0,
    enemySpeed: 0,
    round: 0,
    score: 0,
    changeRounds: undefined,
    sound: undefined,
    myKeys: undefined,
    Emitter: undefined,
    leaderboard: undefined,
    leaderboardMade: false,

    
    // Initializes all variables and functions
    init: function(){
        this.canvas = document.querySelector('canvas');
        this.ctx = this.canvas.getContext('2d');
        this.canvas.width = window.innerWidth;
        this.canvas.height = window.innerHeight;
        this.WIDTH = this.canvas.width;
        this.HEIGHT = this.canvas.height;
        this.spaceSize = 50;
        this.gridWidth = 100;
        this.gridHeight = 100;
        this.playerSize = 50;
        this.castleSize = 50;
        this.castleSprite = new Image();
        this.castleSprite.src = "./media/Proj2Castle.png";
        this.castle = new game.Castle(this.WIDTH/2, this.HEIGHT/2, this.castleSize);
        this.castle.sprite = new this.sprite(this.castle.x, this.castle.y, this.ctx, this.castleSize, this.castleSize, this.castleSprite, 0, 0, 0);
        this.grid = this.makeGrid();
        
        this.playerSprite = new Image();
        this.playerSprite.src = "./media/Proj2DragonSpriteSheet.png";
        this.player = this.initPlayer();
        this.playerMovement = 250;
        
        this.enemySprite = new Image();
        this.enemySprite.src = "./media/Proj2Enemy.png";
        this.bulletList = [];
        this.fireSize = 50;
        this.fireball = new game.Fireball(0, 0, this.fireSize);
        this.round = 1;
        this.changeRounds = false;
        this.score = 0;
        this.leaderboard = [];
        this.initLeaderboard();
        this.enemies = [];
        this.enemySize = 10;
        this.enemySpeed = 50;
        for(var i = 0; i < 10; i++){
            this.spawnEnemies();
            this.enemies[i].setDirection(this.castle);
            this.enemies[i].normalizeDirection();
            this.enemyCount++;
        }

        this.sound.init();  

        this.titleScreenImage = new Image();
        this.gameOverImage = new Image();
        this.titleScreenImage.src = "./media/titlescreen.jpeg";
        this.gameOverImage.src = "./media/gameover.jpg";

        this.gameState = this.GAME_STATE.TITLE_SCREEN;

        this.button = document.querySelector('#startButton');
        var buttonX = this.WIDTH/2;
        buttonX -= 50;
        this.button.style.left = buttonX + "px";
        this.button.addEventListener("click", function(){
            this.sound.playBGAudio();
            this.gameState = this.GAME_STATE.GAME;
            this.button.style.display = "none";
        }.bind(this), false);

        this.canvas.addEventListener("click", function(e){
            this.player.shootFireball(e, this.grid, this.gridWidth, this.gridHeight, this.fireball, this.gameState, this.GAME_STATE, this.checkDist, this.spaceSize, this.sound, this.fireballSound);
        }.bind(this));

        window.onblur = function(){
            this.pauseGame();
        }.bind(this);

        window.onfocus = function(){
            this.resumeGame();
        }.bind(this);

        this.update();
    },
    
    // Runs drawing and audio code
    update: function(){
        this.animationID = requestAnimationFrame(this.update.bind(this));
        
        if(this.paused && this.gameState == this.GAME_STATE.GAME){
            this.drawPause();
        }

        else{
        this.ctx.clearRect(0, 0, this.WIDTH, this.HEIGHT);

        if(this.gameState == this.GAME_STATE.GAME){
            this.drawGame();
        }

        else if(this.gameState == this.GAME_STATE.TITLE_SCREEN){
            this.drawTitle();
        }

        else if(this.gameState == this.GAME_STATE.GAME_OVER){
            this.drawGameOver();
            }
        }
    },
    
    // Draws the game, handles player and enemy movement, castle health, and fireballs
    drawGame: function(){
        this.newWave();
        var deltaTime = this.calculateDeltaTime();
        this.ctx.fillStyle = "#006600";
        this.ctx.fillRect(0, 0, this.WIDTH, this.HEIGHT);
        var prevCount = this.enemyCount;
        for(var i = 0; i < this.enemies.length; i++){
            this.enemyCount = this.enemies[i].checkCollision(this.player, this.castle, this.enemyCount, this.sound, this.eatingSound);
            this.enemies[i].setDirection(this.castle);
            this.enemies[i].normalizeDirection();
            this.enemies[i].move(this.castle, deltaTime);
            this.enemies[i].sprite.update(this.enemies[i].x, this.enemies[i].y, deltaTime);
            if(this.enemies[i].alive){
                this.enemies[i].sprite.render();
            }
            this.enemies[i].arrive(this.castle, this.castleSize);
            if(this.enemyCount < prevCount){
                this.score += 100;
                prevCount = this.enemyCount;
            }
        }
        
        // Draws the shadows
        this.drawGrid();
        this.playerSpriteY = this.player.move(this.playerMovement, deltaTime, this.playerSpriteY);
        this.player.sprite.yIndex = this.playerSpriteY;
        this.castle.sprite.render();
        this.castle.takeDamage(deltaTime, this.sound, this.castleAttackSound);
        if(this.fireball.alpha > 0.0){
            this.fireball.tickLifetime(deltaTime);
        }
        this.fireball.drawFireball(this.ctx);
        this.ctx.globalAlpha = 1;
        this.player.sprite.update(this.player.x, this.player.y, deltaTime);
        this.player.sprite.render();
        this.ctx.fillStyle = "#d9d9d9";
        this.ctx.fillRect(0, 0, this.WIDTH, 40);
        this.ctx.textAlign = "left";
        this.ctx.textBaseline = "top";
        this.ctx.fillStyle = "black";
        this.ctx.font = "20px Cinzel Decorative";
        this.ctx.fillText("Round: " + this.round, 10, 10);
        this.ctx.fillText("Enemies Left: " + this.enemyCount, this.WIDTH/4, 10);
        this.ctx.textAlign = "right";
        this.ctx.fillText("Enemies at Castle: " + this.castle.enemies, this.WIDTH*3/4, 10);
        this.ctx.fillText("Score: " + this.score, this.WIDTH - 20, 10);
        this.ctx.textAlign = "center";
        this.ctx.fillText("Castle Health: " + this.castle.health, (this.WIDTH/2)+(this.castleSize/2), (this.HEIGHT/2)-(this.castleSize/2)-20);
        this.ctx.textAlign = "left";
        if(this.castle.checkDead()){
            this.gameState = this.GAME_STATE.GAME_OVER;
        }
    },

    // Draws title screen
    drawTitle: function(){
        this.ctx.drawImage(this.titleScreenImage, 0, 0, 1280, 853, 0, 0, this.WIDTH, this.HEIGHT);
        this.ctx.fillStyle = 'rgb(51, 51, 51, 0.5)';
        this.ctx.fillRect(0, 0, this.WIDTH, this.HEIGHT);
        this.ctx.textAlign = "center";
        this.ctx.textBaseline = "middle";
        this.ctx.fillStyle = "#d9d9d9";
        this.ctx.font = "30px Cinzel Decorative";
        this.ctx.fillText("Death of the Dragon Lord", this.WIDTH/2, this.HEIGHT/2 - 100);


        this.ctx.font = "20px Cinzel Decorative";
        this.ctx.fillText("A mighty dragon watched as his old lord lay dying. A man with many enemies, age now threatened to take what they never could.", this.WIDTH/2, this.HEIGHT/2);
        this.ctx.fillText("It was no secret that he would die soon. His enemies gathered about his castle and waited until night fell.", this.WIDTH/2, this.HEIGHT/2 + 50);
        this.ctx.fillText("The brave dragon swooped from his perch to make a final stand, knowing that this night would be his lord's last.", this.WIDTH/2, this.HEIGHT/2 + 100);

        this.ctx.font = "30px Cinzel Decorative";
        this.ctx.fillText("Controls", this.WIDTH/2, this.HEIGHT/2 + 200);

        this.ctx.font = "20px Cinzel Decorative";
        this.ctx.fillText("WASD: Move Dragon", this.WIDTH/2, this.HEIGHT/2 + 250);
        this.ctx.fillText("Left Mouse Click: Shoot Fireball", this.WIDTH/2, this.HEIGHT/2 + 300);
        this.ctx.fillText("Hunt down and eat the enemies assaulting your castle. Your dragon's fire cannot hurt them, but it can beat back the darkness.", this.WIDTH/2, this.HEIGHT/2 + 350);
    },

    // Draws game over screen
    drawGameOver: function(){
        // Sets up leaderboard
        if(this.score > 0){
            if(!this.leaderboardMade){
                var i = 0;
                var val = window.localStorage.getItem("key0");
                while(this.score <= val){
                    i++;
                    val = window.localStorage.getItem("key"+i);
                }
                window.localStorage.setItem("key"+i, this.score);
                this.leaderboard[i] = this.score;
                var temp = val;
                var next = 0;
                if(i < 9){
                    next = this.leaderboard[i+1];
                }
                i++;
                while(i < 10){
    
                    window.localStorage.setItem("key"+i, temp);
                    temp = this.leaderboard[i];
                    this.leaderboard[i] = window.localStorage.getItem("key"+i);
                    
                    if(i < 9){
                        next = this.leaderboard[i+1];
                    }
    
                    i++;
                }
                this.leaderboardMade = true;
            }
        }
        
        this.ctx.drawImage(this.gameOverImage, 0, 0, 1024, 680, 0, 0, this.WIDTH, this.HEIGHT);
        this.ctx.fillStyle = 'rgb(51, 51, 51, 0.7)';
        this.sound.stopBGAudio();
        this.ctx.textAlign = "center";
        this.ctx.textBaseline = "middle";
        this.ctx.fillStyle = "#d9d9d9";
        this.ctx.font = "30px Cinzel Decorative";
        this.ctx.fillText("Game Over", this.WIDTH/2, 100);
        this.ctx.fillText("High Scores", this.WIDTH/2, 200);
        for(var i = 0; i < 10; i++){
            var text = i+1 + ". " + this.leaderboard[i];
            this.ctx.fillText(text, this.WIDTH/2, 200 + ((i+1) * 50));
        }
        this.ctx.fillText("Refresh to Play Again", this.WIDTH/2, 800);
    },

    // Draws pause state
    drawPause: function(){
        this.ctx.textAlign = "center";
        this.ctx.textBaseline = "middle";
        this.ctx.fillStyle = "black";
        this.ctx.font = "30px Cinzel Decorative";
        this.ctx.fillText("Paused", this.WIDTH/2, this.HEIGHT/2 - 100);
        if(this.numPauseRect == 0){
            this.ctx.fillStyle = 'rgb(51, 51, 51, 0.5)';
            this.ctx.fillRect(0, 0, this.WIDTH, this.HEIGHT);
            this.numPauseRect++;
        }
    },

    // Initializes the player
    initPlayer: function(){
        var playerObj = new game.Player(this.WIDTH/2, this.HEIGHT/2, this.playerSize);
        playerObj.sprite = new this.sprite(playerObj.x, playerObj.y, this.ctx, this.playerSize, this.playerSize, this.playerSprite, 0.2, 4, this.playerSpriteY);
        return playerObj;
    },

    // Initializes leaderboard from local storage
    initLeaderboard: function(){
        if(window.localStorage.length != 0){
            for(var i = 0; i < 10; i++){
                var key = "key" + i;
                if(window.localStorage.getItem(key) != undefined){
                    this.leaderboard[i] = window.localStorage.getItem(key);
                }

                else{
                    this.leaderboard[i] = 0;
                }
            }
        }

        else{
            for(var i = 0; i < 10; i++){
                var key = "key" + i;
                var value = 0;
                window.localStorage.setItem(key, value);
                this.leaderboard[i] = value;
            }
        }
    },

    // Spawns a single enemy in a random location
    spawnEnemies: function(){
        var enemy = new game.Enemy(Math.random() * this.WIDTH, Math.random() * this.HEIGHT, this.enemySize, this.enemySpeed, this.castle);
        while(Math.abs((enemy.x + (this.enemySize/2)) - (this.castle.x + this.castleSize/2)) < this.enemySize*3 && Math.abs((enemy.y + (this.enemySize/2)) - (this.castle.y + this.castleSize/2)) < this.enemySize*3){
            enemy.x = Math.random() * this.WIDTH;
            enemy.y = Math.random() * this.HEIGHT;
        }
        enemy.sprite = new this.sprite(enemy.x, enemy.y, this.ctx, this.enemySize, this.enemySize, this.enemySprite, 0, 0, 0);
        this.enemies.push(enemy);
    },

    // Initializes the shadow grid
    makeGrid: function(){
        var gameField = [];

        for(var i = 0; i < this.gridWidth; i++){
            for(var j = 0; j < this.gridHeight; j++){
                var space = new game.GridSpace(i * (this.spaceSize/2), j * (this.spaceSize/2));
                if(this.checkDist(this.castle, space, this.playerSize, this.spaceSize)){
                    //space.changeVisibility(true);
                    space.nearCastle = true;
                    this.castle.closeShadows.push(space);
                }
                gameField.push(space);
            }
        }

        return gameField;
    },

    // Draws the shadow grid and handles which shadows will be turned off
    drawGrid: function(){
        for(var i = 0; i < this.grid.length; i++){
            this.checkDist(this.player, this.grid[i], this.playerSize, this.spaceSize);
            this.grid[i].drawSpace(this.ctx, this.spaceSize);
        }
    },

    // Checks the distance between an object (player, castle, or fire) and a given shadow in the shadow grid
    checkDist: function(obj, space, objSize, spaceSize){
        var makeInvisible = false;
        if(space.nearCastle){
            return;
        }

        if(space.nearFire){
            return;
        }
        
        else if(Math.abs((space.x + (spaceSize/2)) - (obj.x + objSize/2)) < spaceSize*3){
            if(Math.abs((space.y + (spaceSize/2)) - (obj.y + objSize/2)) < spaceSize*3){
                makeInvisible = true;
            }
        }
        space.changeVisibility(makeInvisible);
        return makeInvisible;
    },

    // Calculates change in time, borrowed from in-class exercises
    calculateDeltaTime: function(){
		var now,fps;
		now = performance.now(); 
		fps = 1000 / (now - this.lastTime);
		fps = clamp(fps, 12, 60);
		this.lastTime = now; 
		return 1/fps;
    },
    
    // Sets up a new wave once the current one is beaten
    newWave: function(){
        if(this.enemyCount == 0){
            this.round++;
            this.enemySpeed += .2;
            for(var i = 0; i < this.enemies.length; i++){
                this.enemies[i].reInit(this.WIDTH, this.HEIGHT, this.enemySpeed, this.castle, this.castleSize);
                this.enemyCount++;
            }

            for(var j = 0; j < 2; j++){
                this.spawnEnemies();
                
                this.enemyCount++;
            }
        }
    },

    // Handles pausing the game, borrowed from in-class exercises
    pauseGame: function(){
        this.paused = true;
        cancelAnimationFrame(this.animationID);
        //this.stopBGAudio();
        this.update();
    },

    // Handles resuming the game, borrowed from in-class exercises
    resumeGame: function(){
        cancelAnimationFrame(this.animationID);
        this.paused = false;
        this.numPauseRect = 0;
        //this.sound.playBGAudio();
        this.update();
    },

    // Constructor for an object that deals with sprites and spritesheets
    // From tutorial: http://www.williammalone.com/articles/create-html5-canvas-javascript-sprite-animation/
    sprite: function(x, y, ctx, width, height, image, ticksPerFrame, frameCount, yIndex){
        var spriteObj = {};

        spriteObj.x = x;
        spriteObj.y = y;
        spriteObj.ctx = ctx;
        spriteObj.width = width;
        spriteObj.height = height;
        spriteObj.image = image;
        spriteObj.xIndex = 0;
        spriteObj.yIndex = yIndex;
        spriteObj.tickCount = 0;
        spriteObj.ticksPerFrame = ticksPerFrame;
        spriteObj.frameCount = frameCount;

        spriteObj.render = function(){
            spriteObj.ctx.drawImage(spriteObj.image, spriteObj.xIndex * spriteObj.width /*/ spriteObj.frameCount*/, spriteObj.height * spriteObj.yIndex, spriteObj.width, spriteObj.height, spriteObj.x, spriteObj.y, spriteObj.width, spriteObj.height);
        };

        spriteObj.update = function(x, y, deltaTime){
            spriteObj.tickCount += deltaTime;

            spriteObj.x = x;
            spriteObj.y = y;

            if(spriteObj.tickCount > spriteObj.ticksPerFrame){
                spriteObj.tickCount = 0;

                spriteObj.xIndex += 1;

                if(spriteObj.xIndex > frameCount){
                    spriteObj.xIndex = 0;
                }
            }
        }
        return spriteObj;
    }

    
};

// Player module
game.Player = function(){
    function Player(x, y, size){
        this.x = x;
        this.y = y;
        this.size = size;
        this.sprite = undefined;
    }

    // Moves the player with WASD
    Player.prototype.move = function(playerMovement, deltaTime, pastY){
        var spriteYIndex = pastY;
        if(myKeys.keydown[myKeys.KEYBOARD.KEY_W]){
            spriteYIndex = 0;
            if(myKeys.keydown[myKeys.KEYBOARD.KEY_D]){
                this.x += playerMovement * deltaTime;
                this.y -= playerMovement * deltaTime;
            }
            else if(myKeys.keydown[myKeys.KEYBOARD.KEY_A]){
                this.x -= playerMovement * deltaTime;
                this.y -= playerMovement * deltaTime;
            }
            else{
                this.y -= playerMovement * deltaTime;
            }
        }
        else if(myKeys.keydown[myKeys.KEYBOARD.KEY_S]){
            spriteYIndex = 1;
            if(myKeys.keydown[myKeys.KEYBOARD.KEY_D]){
                this.x += playerMovement * deltaTime;
                this.y += playerMovement * deltaTime;
            }
            else if(myKeys.keydown[myKeys.KEYBOARD.KEY_A]){
                this.x -= playerMovement * deltaTime;
                this.y += playerMovement * deltaTime;
            }
            else{
                this.y += playerMovement * deltaTime;
            }
        }
        else if(myKeys.keydown[myKeys.KEYBOARD.KEY_D]){
            spriteYIndex = 3;
            this.x += playerMovement * deltaTime;
        }
        else if(myKeys.keydown[myKeys.KEYBOARD.KEY_A]){
            spriteYIndex = 2;
            this.x -= playerMovement * deltaTime;
        }
        return spriteYIndex;
    }

    // Spawns a fireball at the mouse position
    Player.prototype.shootFireball = function(e, grid, gridWidth, gridHeight, fireball, gameState, GAME_STATE, checkDist, spaceSize, sound, effect){
        if(gameState == GAME_STATE.GAME){
            if(fireball.alpha == 0.0){
                var mouse = getMouse(e);
                fireball.activate(mouse.x, mouse.y);
                fireball.getNearby(grid, gridWidth, gridHeight, spaceSize, checkDist);
                sound.playEffect(effect, 0.3);
            }
        }
    }

    return Player;
}();

//Castle module
game.Castle = function(){
    function Castle(x, y, size){
        this.x = x;
        this.y = y;
        this.health = 100;
        this.timer = 0.5;
        this.closeShadows = [];
        this.enemies = 0;
        this.alive = true;
        this.size = size;
        this.sprite = undefined;
    }

    // Makes nearby shadows invisible
    Castle.prototype.lightShadows = function(ctx){
        for(var i = 0; i < closeShadows.length; i++){
            closeShadows[i].changeVisibility(true);
        }
    }

    // Handles damage dealt by enemies
    Castle.prototype.takeDamage = function(deltaTime, sound, effect){
        if(this.enemies > 0){
            this.timer -= deltaTime;
            if(this.timer <= 0.0){
                this.health -= this.enemies;
                this.timer = 0.5;
                sound.playEffect(effect, 0.1);
            }
        }
        if(this.enemies == 0){
            this.timer = 0.5;
        }
    }

    // Checks to see if the castle is out of health
    Castle.prototype.checkDead = function(){
        if(this.health <= 0){
            return true;
        }
        return false;
    }

    return Castle;
}();

//Grid Space module
game.GridSpace = function(){
    function GridSpace(x, y){
            this.x = x;
            this.y = y;
            this.alpha = 1.0;
            this.nearCastle = false;
            this.nearFire = false;
    }

    // Draws the shadow grid space with specified alpha
    GridSpace.prototype.drawSpace = function(ctx, spaceSize){
        ctx.fillStyle = '#333333';
        ctx.globalAlpha = this.alpha;
        ctx.fillRect(this.x, this.y, spaceSize, spaceSize);
    }

    // Changes visibility setting, true parameter to make invisible
    GridSpace.prototype.changeVisibility = function(makeInvisible){
        if(makeInvisible){
            this.alpha = 0;
            return;
        }
        this.alpha = 1;
    }

    return GridSpace;
}();

// Fireball module
game.Fireball = function(){
    function Fireball(x, y, size){
            this.x = x;
            this.y = y;
            this.size = size;
            this.alpha = 0.0;
            this.closeShadows = [];
            this.time = 5.0;
            this.emitter = new game.Emitter();
            this.active = false;
    }

    // Draws fireball with a particle effect
    Fireball.prototype.drawFireball = function(ctx){
        var emitterPoint = {
            x: this.x,
            y: this.y
        };
        if(this.active){
            this.emitter.updateAndDraw(ctx, emitterPoint);
        }
        //ctx.fillStyle = 'orange';
        ctx.globalAlpha = this.alpha;
        //ctx.fillRect(this.x, this.y, this.size, this.size);
    }

    // Makes fire appear and effect shadows around it
    Fireball.prototype.activate = function(x, y){
        this.active = true;
        this.emitter.createParticles({x, y});
        this.alpha = 1.0;
        this.time = 5.0;
        this.x = x;
        this.y = y;
    }

    // Counts down, then deactivates fire
    Fireball.prototype.tickLifetime = function(deltaTime){
        this.time -= deltaTime;
        if(this.time <= 0.0){
            this.active = false;
            this.alpha = 0.0;
            this.removeNearby();
        }
    }

    // Gets nearby grid spaces
    Fireball.prototype.getNearby = function(grid, gridWidth, gridHeight, spaceSize, checkDist){
        for(var i = 0; i < grid.length; i++){
            if(checkDist(this, grid[i], this.size, spaceSize)){
                this.closeShadows.push(grid[i]);
                grid[i].nearFire = true;
            }
        }
    }

    // Removes the nearby spaces from the list
    Fireball.prototype.removeNearby = function(){
        for(var i = 0; i < this.closeShadows.length; i++){
            this.closeShadows[i].changeVisibility(false);
            this.closeShadows[i].nearFire = false;
            //this.closeShadows.splice(0, 1);
        }
        this.closeShadows = [];
    }

    return Fireball;
}();

// Enemy module
game.Enemy = function(){
    function Enemy(x, y, size, speed){
        this.x = x;
        this.y = y;
        this.size = size;
        this.speed = speed;
        this.direction = {
            x: 0,
            y: 0,
            length: 0
        };
        this.atCastle = false;
        this.alive = true;
        this.sprite = undefined;
    }

    // Repositions and revives enemy for the next wave
    Enemy.prototype.reInit = function(width, height, speed, castle, castleSize){
        this.x = Math.random() * width;
        this.y = Math.random() * height;
        this.speed = speed;
        this.alive = true;
        this.atCastle = false;
        while(Math.abs((this.x + (this.size/2)) - (castle.x + castleSize/2)) < this.size*3 && Math.abs((this.y + (this.size/2)) - (castle.y + castleSize/2)) < this.size*3){
            this.x = Math.random() * width;
            this.y = Math.random() * height;
        }
    }

    // Set's direction vector toward castle
    Enemy.prototype.setDirection = function(castle){
        this.direction.x = castle.x - this.x;
        this.direction.y = this.y - castle.y;
        this.direction.length = Math.sqrt(this.direction.x*this.direction.x + this.direction.y*this.direction.y);
    }

    // Normalizes direction vector
    Enemy.prototype.normalizeDirection = function(){
        this.direction.x /= this.direction.length;
        this.direction.y /= this.direction.length;
    }

    // Moves enemy
    Enemy.prototype.move = function(castle, deltaTime){
        if(!this.atCastle){
            this.x += this.direction.x * this.speed * deltaTime;
            this.y -= this.direction.y * this.speed * deltaTime;

            this.direction.x = castle.x - this.x;
            this.direction.y = castle.y - this.y;
            this.direction.length = Math.sqrt(this.direction.x*this.direction.x + this.direction.y*this.direction.y);
        }
    }

    // Stops enemy and adds enemy to castle's enemy list when they arrive at the castle
    Enemy.prototype.arrive = function(castle, castleSize){
        if(!this.atCastle && this.alive){
            if(Math.abs((castle.x + castleSize/2) - (this.x + this.size/2)) <= 35 && Math.abs((castle.y + castleSize/2) - (this.y + this.size/2)) <= 35){
                this.atCastle = true;
                castle.enemies++;
            }
        }
    }

    // Checks to see if the player has collided with enemy, makes enemy invisible and removes enemy from castle's enemy list if they have
    Enemy.prototype.checkCollision = function(player, castle, enemyCount, sound, eatingSound){
        if(this.alive){
            if(this.x < player.x + player.size){
                if(this.x + this.size > player.x){
                    if(this.y < player.y + player.size){
                        if(this.y + this.size > player.y){
                            if(this.atCastle){
                                castle.enemies--;
                                this.atCastle = false;
                            }
                            this.alive = false;
                            enemyCount--;
                            sound.playEffect(eatingSound, 0.05);
                        }
                    }
                }
            }
        }
        return enemyCount;
    }

    return Enemy;
}();

// Sound manager, borrowed from in-class exercises
game.sound = (function(){
	var bgAudio = undefined;
	
	function init(){
		bgAudio = document.querySelector("#bgAudio");
		bgAudio.volume=0.25;
	}
		
	function stopBGAudio(){
		bgAudio.pause();
		bgAudio.currentTime = 0;
	}
	
	function playEffect(effect, volume){
    	var effectSound = document.createElement('audio');
    	effectSound.volume = volume;
		effectSound.src = "media/" + effect;
		effectSound.play();
	}

	function playBGAudio(){
		bgAudio.play();
	}

	return{
		init: init,
		stopBGAudio: stopBGAudio,
		playEffect: playEffect,
		playBGAudio: playBGAudio
	};
}());

// Particle system module, borrowed from in-class exercises
game.Emitter=function(){

	function Emitter(){
		// public
		this.numParticles = 55;
		this.useCircles = false;
		this.useSquares = true;
		this.xRange = 2;
		this.yRange = 10;
		this.minXspeed = -0.5;
		this.maxXspeed = 0.5;
		this.minYspeed = -0.5;
		this.maxYspeed = -2;
		this.startRadius = 4;
		this.expansionRate = 0.2;
		this.decayRate = 1.5;
		this.lifetime = 100;
		this.red = 255;
		this.green = 102;
		this.blue = 0;
		
		// private
		this._particles = undefined;
	};
	
	
	// "public" methods
	var p=Emitter.prototype;
	
	p.createParticles = function(emitterPoint){
		// initialize particle array
		this._particles = [];
				
		// create exhaust particles
		for(var i=0; i< this.numParticles; i++){
			// create a particle object and add to array
			var p = {};
			this._particles.push(_initParticle(this, p, emitterPoint));
		}
	};
	
	p.updateAndDraw = function(ctx, emitterPoint){
			/* move and draw particles */
			// each frame, loop through particles array
			// move each particle down screen, and slightly left or right
			// make it bigger, and fade it out
			// increase its age so we know when to recycle it
			
			for(var i=0;i<this._particles.length;i++){
				var p = this._particles[i];
							
				p.age += this.decayRate;
				p.r += this.expansionRate;
				p.x += p.xSpeed
				p.y += p.ySpeed
				var alpha = 1 - p.age/this.lifetime;
				
				if(this.useSquares){
					// fill a rectangle	
					ctx.fillStyle = "rgba(" + this.red + "," + this.green + "," + 			
					this.blue + "," + alpha + ")"; 
					ctx.fillRect(p.x, p.y, p.r, p.r);
					// note: this code is easily modified to draw images
				}
				
				if(this.useCircles){
					// fill a circle
					ctx.fillStyle = "rgba(" + this.red + "," + this.green + "," + 			
					this.blue + "," + alpha + ")"; 
			
					ctx.beginPath();
					ctx.arc(p.x, p.y, p.r, Math.PI * 2, false);
					ctx.closePath();
					ctx.fill();
				}
							
				// if the particle is too old, recycle it
				if(p.age >= this.lifetime){
					_initParticle(this, p, emitterPoint);
				}		
			} // end for loop of this._particles
	} // end updateAndDraw()
			
	// "private" method
	function _initParticle(obj, p, emitterPoint){
		
		// give it a random age when first created
		p.age = getRandom(0,obj.lifetime);
				
		p.x = emitterPoint.x + getRandom(-obj.xRange, obj.xRange);
		p.y = emitterPoint.y + getRandom(0, obj.yRange);
		p.r = getRandom(obj.startRadius/2, obj.startRadius); // radius
		p.xSpeed = getRandom(obj.minXspeed, obj.maxXspeed);
		p.ySpeed = getRandom(obj.minYspeed, obj.maxYspeed);
		return p;
	};
	
	
	return Emitter;
}();